import jas.engine.*;
import jas.events.*;
import jas.plot.*;
import jas.space.*;

import jas.net.*;
import jas.io.*;

import java.util.*;
import java.io.File;

/**
 * An iteractive simulation of the Prisoner's Dilemma game, based on the
 * Sim2Web architecture by Sergio Margarita (sergio.margarita@unito.it)
 * and Michele Sonnessa (sonnessa@di.unito.it). <br>
 * See the article at ... for more details.
 *
 * @author Michele Sonnessa
 */

public class TitForTat extends SimModel implements ISimEventListener
{
  public static final int STRATEGY_COOPERATE = 0;
  public static final int STRATEGY_DEFEAT = 1;

  /** The XML-RPC server */
  private InformationServer server;
  /** The current list of players */
  private ArrayList list;

  /** The TCP port used by the XML-RPC server.*/
  public int infoServerTCPPort = 9990;

  /** If the flag is set to true the simulation does not activate the web
   *  interface.*/
  public boolean runLocally = false;

  /** The data table shown on the screen, containing the list of allowded players. */
  private DataTable playerList;
  /** The pay off matrix*/
  private DataTable payOffMatrix;

  /** Intializations.*/
  public void setParameters()
  {
    /* Create the player list window using the field definition contained
       by the parameter form file.*/
    playerList = new DataTable(new File(path + "PlayerList.pForm.xml"));
    playerList.addRow();
    playerList.setValueAt("michele", 0, 0);
    playerList.addRow();
    playerList.setValueAt("sergio", 1, 0);
    // Show the table on the screen.
    DataTableFrame playerListFrame = new DataTableFrame(playerList, "Player list");
    addSimWindow(playerListFrame);

    // Create the payoff matrix structure
    payOffMatrix = new DataTable();
    // Let's define the columns
    payOffMatrix.addColumn("Cooperate", ParameterField.TYPE_DBL);
    payOffMatrix.addColumn("Defeat", ParameterField.TYPE_DBL);
    payOffMatrix.addRow();
    payOffMatrix.setValueAt(new Double(80), 0 , 0);
    payOffMatrix.setValueAt(new Double(-100), 0, 1);
    payOffMatrix.addRow();
    payOffMatrix.setValueAt(new Double(100), 1, 0);
    payOffMatrix.setValueAt(new Double(-60), 1, 1);
    DataTableFrame payOffFrame = new DataTableFrame(payOffMatrix, "Pay off matrix");
    payOffFrame.setRowNumberModifiable(false); //Lock the add and remove row buttons.
    addSimWindow(payOffFrame);

    //Open the main parameter form.
    ParametersBag bag = new ParametersBag(path + "Rewards.pForm.xml");
    ParameterFrame pF = new ParameterFrame(this, "Model parameters", bag);
    addSimWindow(pF);
  }

  /** Build model into memory.*/
  public void buildModel()
  {
    buildServer();
    buildObjects();
    buildActions();
  }

  /** Build up the XML-RPC server.*/
  private void buildServer()
  {
    if (runLocally)
      return;

    server = new InformationServer(infoServerTCPPort);
    System.out.println("XML-RPC server has been installed on the " + infoServerTCPPort
                       + " TCP/IP port.");
  }

  /** Refresh the list of published data.*/
  public void updateServer()
  {
    if (server == null)
      return;

    // Iterate the list of players and ask each of them their state.
    Iterator it = list.iterator();
    while (it.hasNext())
    {
      TitForTatBug bug = (TitForTatBug) it.next();

      server.addInfo(bug.getName(), "id01", "Cash: " + bug.getCash() + "");
      server.addInfo(bug.getName(), "id02", getStrategyString(bug.getStrategy()));
      server.addInfo(bug.getName(), "id03", bug.getDecisionsHistory());
      server.addInfo(bug.getName(), "id04", bug.getSVG());
    }
  }

  private String getStrategyString(int coop)
  {
    if (coop == STRATEGY_COOPERATE)
      return "Cooperate";
    else
      return "Defeat";
  }

  /** Builds the players.*/
  private void buildObjects()
  {
    list = new ArrayList();

	if ((playerList.getRows().size() % 2) != 0)
	{
		System.err.println("The number of player must be pair!");
		System.exit(1);
	}

    for (int i = 0; i < playerList.getRows().size(); i++)
    {
      TitForTatBug bug = new TitForTatBug(playerList.getValueAt(i, 0).toString(),
                            server);
      //If runs locally the strategy is decided randomly.
      if (runLocally)
        bug.receiveOrder(Sim.getRnd().getIntFromTo(STRATEGY_COOPERATE, STRATEGY_DEFEAT));
      list.add(bug);
    }
  }

  /** Updates each player's cash according to the pay off matrix.*/
  public void step()
  {
    TitForTatBug bug1, bug2;
    int strategy1, strategy2;

    for (int i = 0; i < list.size(); i += 2)
    {
      bug1 = (TitForTatBug) list.get(i);
      bug2 = (TitForTatBug) list.get(i + 1);

      if (!bug1.areYouActive() && !bug2.areYouActive() && !runLocally)
        return;

      strategy1 = bug1.getStrategy();
      strategy2 = bug2.getStrategy();

      bug1.update(((Double)payOffMatrix.getValueAt(strategy1, strategy2)).doubleValue());
      bug2.update(((Double)payOffMatrix.getValueAt(strategy2, strategy1)).doubleValue());
    }
  }

  /** Builds the schedule.*/
  private void buildActions()
  {
    SimGroupEvent s;

    s = eventList.scheduleGroup(0, (int) Sim.toSecs(10));
    s.addCollectionEvent(list, Sim.EVENT_STEP);
    s.addEvent(this, Sim.EVENT_STEP);

    eventList.scheduleSimple(0, (int) Sim.toSecs(10), this, "updateServer");

  }

 /** Implementation of ISimEventListener interface.*/
  public void performAction(int actionID)
  {
    if (actionID == Sim.EVENT_STEP)
      step();
  }

  public static void main(String[] args)
  {

	TitForTat m = new TitForTat();

    SimEngine eng = new SimEngine();

    eng.addModel(m);
    eng.startGUI(false);

    eng.buildModels();
    eng.start();
  }

}
