
import jas.engine.*;
import jas.space.*;
import jas.net.*;
import jas.plot.*;
import jas.io.SVGGenerator;

import java.util.*;
import java.awt.Point;
import jas.events.ISimEventListener;
import jas.stats.IDblSource;

/**
 * An iteractive simulation of the Prisoner's Dilemma game, based on the
 * Sim2Web architecture by Sergio Margarita (sergio.margarita@unito.it)
 * and Michele Sonnessa (sonnessa@di.unito.it). <br>
 * See the article at ... for more details.
 *
 * @author Michele Sonnessa
 */

public class TitForTatBug implements ISimEventListener, IDblSource
{
  /** Used by getDoubleValue function.*/
  public static final int VALUE_CASH = 0;

  /** A reference to the XML-RPC server. */
  private InformationServer srv;
  /** It is set to true when the agent receives the first order. */
  private boolean commandReceived = false;
  /** The current strategy.*/
  private int strategy;
  /** User/agent's name.*/
  private String name;
  /** Current cash amount.*/
  private double cash;

  /** A step counter used to format the history string: every 5 chars it is
   *  add a <br> tag.*/
  private int counter = 0;

  /** The player's personal plotter.*/
  private TimePlot plotter;

  /** A string representing the sequence of the past strategies.*/
  private StringBuffer decisionsHistory;

  /** Constructor.
   * @param name The unique name of the player.
   * @param srv A reference to the XML-RPC. May be null.*/
  public TitForTatBug(String name, InformationServer srv)
  {
    this.srv = srv;
    this.name = name;
    strategy = TitForTat.STRATEGY_DEFEAT;
    cash = 0;
    decisionsHistory = new StringBuffer(50);

    plotter = new TimePlot(name);
    plotter.addSeries("cash", this, VALUE_CASH);
    Sim.engine.addSimWindow((TitForTat) Sim.engine.getModelWithID("TitForTat"),
                        plotter);

    if (srv != null)
      plotter.setVisible(false);
  }

  /** @return True if player has received the first order.*/
  public boolean areYouActive() { return commandReceived; }

  /** @return The player's name.*/
  public String getName() { return name; }

  /** Sends an order to the player about which strategy to use.
   * @param order One of the TitForTat.STRATEGY_ constants. */
  public void receiveOrder(int order)
  {
    commandReceived = true;
    strategy = order;
  }

  /** @return The current strategy. */
  public int getStrategy() { return strategy; }

  /** @return The current cash amount. */
  public double getCash() { return cash; }

  /** Checks if there are external orders. It works only when a XML-RPC is available. */
  public void step()
  {
    String s = null;
    if (srv != null)
      s = srv.receiveInfo(name, "id06");
    if (s == null)
      return;

    if (s.equals("C"))
      receiveOrder(TitForTat.STRATEGY_COOPERATE);
    else
      receiveOrder(TitForTat.STRATEGY_DEFEAT);
  }

  /** Update the cash amount.
   * @param amount The money to add or subtract to the cash.*/
  public void update(double amount)
  {
    if (!commandReceived)
      return;

    if (++counter == 5)
    {
      decisionsHistory.append("<br>");
      counter = 0;
    }

    cash += amount;
    if (strategy == TitForTat.STRATEGY_COOPERATE)
      decisionsHistory.append('C');
    else
      decisionsHistory.append('T');
    System.out.println(name + " = " + cash);

    plotter.update();
    plotter.getPlot().fillPlot();
  }

  /** @return A SVG representation of the plotter status.*/
  public String getSVG()
  {
    return SVGGenerator.generateAsPicture(plotter.getPlot());
  }

  /** @return A string representing the past strategies.*/
  public String getDecisionsHistory()
  {
    return decisionsHistory.toString();
  }

  /** Implementation of ISimEventListener interface.
   * @param actionID The only avaliable action is EVENT_STEP.*/
  public void performAction(int actionID)
  {
    if (actionID == Sim.EVENT_STEP)
      step();
  }

  /** Implementation of IDblSource interface.
   * @param valueID The only avaliable value is VALUE_CASH.
   * @return The value requested.*/
  public double getDoubleValue(int valueID)
  {
    if (valueID == VALUE_CASH)
      return cash;

    return 0.0;
  }

}
